@@ -16,12 +16,16 @@ import time
 import yaml
 
 from blazarclient import client as blazar_client
+from cinderclient import client as cinder_client
 from glanceclient.v2 import client as glance_client
 from keystoneauth1.identity import v3
 from keystoneauth1 import session
+from keystoneclient import adapter
 from neutronclient.v2_0 import client as neutron_client
 from novaclient import client as nova_client
 from oslo_config import cfg
+from oslo_log import log as logging
+from oslo_serialization import jsonutils
 from tempest.lib import base
 
 from tacker.plugins.common import constants as evt_constants
@@ -30,12 +34,54 @@ from tacker.tests.functional import clients
 from tacker.tests.utils import read_file
 from tacker import version
 
+from tackerclient.common import exceptions
 from tackerclient.v1_0 import client as tacker_client
 
 
+LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 
 
+class SessionClient(adapter.Adapter):
+    def request(self, *args, **kwargs):
+        kwargs.setdefault('authenticated', False)
+        kwargs.setdefault('raise_exc', False)
+
+        content_type = kwargs.pop('content_type', None) or 'application/json'
+
+        headers = kwargs.setdefault('headers', {})
+        headers.setdefault('Accept', content_type)
+
+        try:
+            kwargs.setdefault('data', kwargs.pop('body'))
+        except KeyError:
+            pass
+
+        if kwargs.get('data'):
+            headers.setdefault('Content-Type', content_type)
+
+        return super(SessionClient, self).request(*args, **kwargs)
+
+    def _decode_json(self, response):
+        body = response.text
+        if body and response.headers['Content-Type'] == 'text/plain':
+            return body
+        elif body and response.headers['Content-Type'] == 'text/x-sh':
+            return body
+        elif body:
+            return jsonutils.loads(body)
+        else:
+            return ""
+
+    def do_request(self, url, method, **kwargs):
+        kwargs.setdefault('authenticated', True)
+        resp = self.request(url, method, **kwargs)
+        if resp.headers['Content-Type'] == 'application/zip':
+            return resp, resp.content
+        body = self._decode_json(resp)
+        return resp, body
+
+
 class BaseTackerTest(base.BaseTestCase):
     """Base test case class for all Tacker API tests."""
 
@@ -50,7 +96,10 @@ class BaseTackerTest(base.BaseTestCase):
                  **kwargs)
 
         cls.client = cls.tackerclient()
+        cls.http_client = cls.tacker_http_client()
         cls.h_client = cls.heatclient()
+        cls.glance_client = cls.glanceclient()
+        cls.cinder_client = cls.cinderclient()
 
     @classmethod
     def get_credentials(cls):
@@ -59,9 +108,10 @@ class BaseTackerTest(base.BaseTestCase):
         return vim_params
 
     @classmethod
-    def tackerclient(cls):
+    def get_auth_session(cls):
         vim_params = cls.get_credentials()
-        auth = v3.Password(auth_url=vim_params['auth_url'],
+        auth = v3.Password(
+            auth_url=vim_params['auth_url'],
             username=vim_params['username'],
             password=vim_params['password'],
             project_name=vim_params['project_name'],
@@ -69,7 +119,19 @@ class BaseTackerTest(base.BaseTestCase):
             project_domain_name=vim_params['project_domain_name'])
         verify = 'True' == vim_params.pop('cert_verify', 'False')
         auth_ses = session.Session(auth=auth, verify=verify)
-        return tacker_client.Client(session=auth_ses)
+        return auth_ses
+
+    @classmethod
+    def tacker_http_client(cls):
+        auth_session = cls.get_auth_session()
+        return SessionClient(session=auth_session,
+                             service_type='nfv-orchestration',
+                             region_name='RegionOne')
+
+    @classmethod
+    def tackerclient(cls):
+        auth_session = cls.get_auth_session()
+        return tacker_client.Client(session=auth_session, retries=5)
 
     @classmethod
     def novaclient(cls):
@@ -135,19 +197,46 @@ class BaseTackerTest(base.BaseTestCase):
         auth_ses = session.Session(auth=auth, verify=verify)
         return glance_client.Client(session=auth_ses)
 
+    @classmethod
+    def aodh_http_client(cls):
+        auth_session = cls.get_auth_session()
+        return SessionClient(session=auth_session,
+                             service_type='alarming',
+                             region_name='RegionOne')
+
+    @classmethod
+    def cinderclient(cls):
+        vim_params = cls.get_credentials()
+        auth = v3.Password(auth_url=vim_params['auth_url'],
+            username=vim_params['username'],
+            password=vim_params['password'],
+            project_name=vim_params['project_name'],
+            user_domain_name=vim_params['user_domain_name'],
+            project_domain_name=vim_params['project_domain_name'])
+        verify = 'True' == vim_params.pop('cert_verify', 'False')
+        auth_ses = session.Session(auth=auth, verify=verify)
+        return cinder_client.Client(constants.CINDER_CLIENT_VERSION,
+                                    session=auth_ses)
+
     def get_vdu_resource(self, stack_id, res_name):
         return self.h_client.resources.get(stack_id, res_name)
 
     def wait_until_vnf_status(self, vnf_id, target_status, timeout,
                               sleep_interval):
         start_time = int(time.time())
+        status = None
         while True:
+            try:
                 vnf_result = self.client.show_vnf(vnf_id)
                 status = vnf_result['vnf']['status']
-                if (status == target_status) or (
-                        (int(time.time()) - start_time) > timeout):
+                if status == target_status:
                     break
-                time.sleep(sleep_interval)
+            except exceptions.InternalServerError:
+                pass
+
+            if int(time.time()) - start_time > timeout:
+                break
+            time.sleep(sleep_interval)
 
         self.assertEqual(status, target_status,
                          "vnf %(vnf_id)s with status %(status)s is"
@@ -170,18 +259,24 @@ class BaseTackerTest(base.BaseTestCase):
                                   constants.VNF_CIRROS_CREATE_TIMEOUT,
                                   constants.ACTIVE_SLEEP_TIME)
 
-    def wait_until_vnf_delete(self, vnf_id, timeout):
+    def wait_until_vnf_delete(self, vnf_id, timeout, sleep_interval=1):
         start_time = int(time.time())
         while True:
+            status = None
             try:
                 vnf_result = self.client.show_vnf(vnf_id)
-                time.sleep(1)
-            except Exception:
+                status = vnf_result['vnf']['status']
+            except exceptions.NotFound:
                 return
-            status = vnf_result['vnf']['status']
-            if (status != 'PENDING_DELETE') or ((
-                    int(time.time()) - start_time) > timeout):
+            except Exception as e:
+                LOG.error("Failed to get vnf status: %s", str(e))
+
+            if status is not None and status != 'PENDING_DELETE':
                 raise Exception("Failed with status: %s" % status)
+            if int(time.time()) - start_time > timeout:
+                raise Exception("Timeout for deleting vnf %s.",
+                                vnf_id)
+            time.sleep(sleep_interval)
 
     def wait_until_vnf_dead(self, vnf_id, timeout, sleep_interval):
         self.wait_until_vnf_status(vnf_id, 'DEAD', timeout,
@@ -191,8 +286,9 @@ class BaseTackerTest(base.BaseTestCase):
         self.assertIsNotNone(vnf_instance)
         self.assertIsNotNone(vnf_instance['vnf']['id'])
         self.assertIsNotNone(vnf_instance['vnf']['instance_id'])
-        self.assertEqual(vnf_instance['vnf']['vnfd_id'], vnfd_instance[
-            'vnfd']['id'])
+        if vnfd_instance:
+            self.assertEqual(vnf_instance['vnf']['vnfd_id'], vnfd_instance[
+                'vnfd']['id'])
 
     def verify_vnf_restart(self, vnfd_instance, vnf_instance):
         vnf_id = vnf_instance['vnf']['id']
@@ -279,3 +375,75 @@ class BaseTackerTest(base.BaseTestCase):
                             }
                 }
         self.client.post('/vnfs/%s/triggers' % vnf, body)
+
+    def assertDictSupersetOf(self, expected_subset, actual_superset):
+        """Checks that actual dict contains the expected dict.
+
+        After checking that the arguments are of the right type, this checks
+        that each item in expected_subset is in, and matches, what is in
+        actual_superset. Separate tests are done, so that detailed info can
+        be reported upon failure.
+        """
+        if not isinstance(expected_subset, dict):
+            self.fail("expected_subset (%s) is not an instance of dict" %
+                      type(expected_subset))
+        if not isinstance(actual_superset, dict):
+            self.fail("actual_superset (%s) is not an instance of dict" %
+                      type(actual_superset))
+        for k, v in expected_subset.items():
+            self.assertIn(k, actual_superset)
+            self.assertEqual(v, actual_superset[k],
+                             "Key %(key)s expected: %(exp)r, actual %(act)r" %
+                             {'key': k, 'exp': v, 'act': actual_superset[k]})
+
+    def create_cinder_volume(cls, vol_size, vol_name):
+        try:
+            cinder_volume = cls.cinder_client.volumes.create(vol_size,
+                    name=vol_name)
+        except Exception as e:
+            LOG.error("Failed to create cinder volume: %s", str(e))
+            return None
+
+        return cinder_volume.id
+
+    def delete_cinder_volume(cls, vol_id):
+        try:
+            cls.cinder_client.volumes.delete(vol_id)
+        except Exception as e:
+            LOG.error("Failed to delete cinder volume: %s", str(e))
+
+    def vnfd_and_vnf_create(self, vnfd_file, vnf_name, volume_id=None,
+            volume_name=None):
+        input_yaml = read_file(vnfd_file)
+        tosca_dict = yaml.safe_load(input_yaml)
+        if volume_id is not None:
+            volume_detail = tosca_dict['topology_template']['inputs']
+            volume_detail[volume_name]['default'] = volume_id
+        tosca_arg = {'vnfd': {'name': vnf_name,
+                              'attributes': {'vnfd': tosca_dict}}}
+
+        # Create vnfd with tosca template
+        vnfd_instance = self.client.create_vnfd(body=tosca_arg)
+        self.assertIsNotNone(vnfd_instance)
+
+        # Create vnf with vnfd_id
+        vnfd_id = vnfd_instance['vnfd']['id']
+        self.addCleanup(self.client.delete_vnfd, vnfd_id)
+
+        vnf_arg = {'vnf': {'vnfd_id': vnfd_id, 'name': vnf_name}}
+        vnf_instance = self.client.create_vnf(body=vnf_arg)
+        self.validate_vnf_instance(vnfd_instance, vnf_instance)
+
+        return vnfd_instance, vnf_instance, tosca_dict
+
+    def vnfd_and_vnf_create_inline(self, vnfd_file, vnf_name):
+        vnfd_instance = {}
+        input_yaml = read_file(vnfd_file)
+        tosca_dict = yaml.safe_load(input_yaml)
+
+        # create vnf directly from template
+        vnf_arg = {'vnf': {'vnfd_template': tosca_dict, 'name': vnf_name}}
+        vnf_instance = self.client.create_vnf(body=vnf_arg)
+        self.validate_vnf_instance(vnfd_instance, vnf_instance)
+
+        return vnf_instance, tosca_dict
