@@ -13,29 +13,68 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+import ddt
+from unittest import mock
+
+from oslo_utils import uuidutils
+
 from tacker.common import exceptions
 from tacker import context
+from tacker.db import api as sqlalchemy_api
+from tacker.db.nfvo import nfvo_db
 from tacker import objects
 from tacker.tests.unit.db.base import SqlTestCase
 from tacker.tests.unit.objects import fakes
+from tacker.tests.unit.vnflcm import fakes as fakes_vnflcm
 from tacker.tests import uuidsentinel
 
-from tacker.db import api as sqlalchemy_api
-
 get_engine = sqlalchemy_api.get_engine
 
 
+class FakeApiModelQuery:
+
+    def __init__(
+            self,
+            callback_filter_by=None,
+            callback_update=None,
+            callback_options=None):
+        self.callback_filter_by = callback_filter_by
+        self.callback_update = callback_update
+        self.callback_options = callback_options
+
+    def options(self, *args, **kwargs):
+        if self.callback_options:
+            self.callback_options(*args, **kwargs)
+
+    def filter_by(self, *args, **kwargs):
+        if self.callback_filter_by:
+            self.callback_filter_by(*args, **kwargs)
+        return self
+
+    def update(self, *args, **kwargs):
+        if self.callback_update:
+            self.callback_update(*args, **kwargs)
+        return self
+
+
+@ddt.ddt
 class TestVnfInstance(SqlTestCase):
 
+    maxDiff = None
+
     def setUp(self):
         super(TestVnfInstance, self).setUp()
         self.context = context.get_admin_context()
         self.vnf_package = self._create_and_upload_vnf_package()
+        self.vims = nfvo_db.Vim(**fakes.vim_data)
         self.engine = get_engine()
         self.conn = self.engine.connect()
+        self.body_data = self._create_body_data()
+        self.vnfd_pkg_data = self._create_vnfd_pkg_data()
+        self.vim = nfvo_db.Vim()
 
-    def _create_and_upload_vnf_package(self):
+    @mock.patch.object(objects.VnfPackageVnfd, 'create')
+    def _create_and_upload_vnf_package(self, mock_create):
         vnf_package = objects.VnfPackage(context=self.context,
                                          **fakes.vnf_package_data)
         vnf_package.create()
@@ -43,6 +82,7 @@ class TestVnfInstance(SqlTestCase):
         vnf_pack_vnfd = fakes.get_vnf_package_vnfd_data(
             vnf_package.id, uuidsentinel.vnfd_id)
 
+        mock_create.return_value = fakes.return_vnf_package_vnfd_data()
         vnf_pack_vnfd_obj = objects.VnfPackageVnfd(
             context=self.context, **vnf_pack_vnfd)
         vnf_pack_vnfd_obj.create()
@@ -52,6 +92,32 @@ class TestVnfInstance(SqlTestCase):
 
         return vnf_pack_vnfd_obj
 
+    def _create_body_data(self):
+        body_data = {}
+        body_data['vnf_instance_name'] = "new_instance_name"
+        body_data['vnf_instance_description'] = "new_instance_discription"
+        body_data['vnfd_id'] = "2c69a161-0000-4b0f-bcf8-391f8fc76600"
+        body_data['vnf_configurable_properties'] = {"test": "test_value"}
+        body_data['vnfc_info_modifications_delete_ids'] = ["test1"]
+        body_data['vnf_pkg_id'] = uuidsentinel.vnf_pkg_id
+        return body_data
+
+    def _create_vnfd_pkg_data(self):
+        vnfd_pkg_data = {}
+        vnfd_pkg_data['vnf_provider'] =\
+            fakes.return_vnf_package_vnfd_data().get('vnf_provider')
+        vnfd_pkg_data['vnf_product_name'] =\
+            fakes.return_vnf_package_vnfd_data().get('vnf_product_name')
+        vnfd_pkg_data['vnf_software_version'] =\
+            fakes.return_vnf_package_vnfd_data().get('vnf_software_version')
+        vnfd_pkg_data['vnfd_version'] =\
+            fakes.return_vnf_package_vnfd_data().get('vnfd_version')
+        vnfd_pkg_data['package_uuid'] =\
+            fakes.return_vnf_package_vnfd_data().get('package_uuid')
+        vnfd_pkg_data['vnfd_id'] =\
+            fakes.return_vnf_package_vnfd_data().get('vnfd_id')
+        return vnfd_pkg_data
+
     def test_create(self):
         vnf_instance_data = fakes.get_vnf_instance_data(
             self.vnf_package.vnfd_id)
@@ -130,6 +196,19 @@ class TestVnfInstance(SqlTestCase):
         self.assertTrue(result.objects, list)
         self.assertTrue(result.objects)
 
+    def test_vnf_instance_list_get_by_filters(self):
+        vnf_instance_data = fakes.get_vnf_instance_data(
+            self.vnf_package.vnfd_id)
+        vnf_instance = objects.VnfInstance(context=self.context,
+                                           **vnf_instance_data)
+        vnf_instance.create()
+        filters = {'field': 'instantiation_state', 'model': 'VnfInstance',
+                   'value': 'NOT_INSTANTIATED',
+                   'op': '=='}
+        vnf_instance_list = objects.VnfInstanceList.get_by_filters(
+            self.context, filters=filters)
+        self.assertEqual(1, len(vnf_instance_list))
+
     @mock.patch('tacker.objects.vnf_instance._destroy_vnf_instance')
     def test_destroy(self, mock_vnf_destroy):
         vnf_instance_data = fakes.get_vnf_instance_data(
@@ -144,3 +223,24 @@ class TestVnfInstance(SqlTestCase):
         vnf_instance_obj = objects.VnfInstance(context=self.context)
         self.assertRaises(exceptions.ObjectActionError,
                           vnf_instance_obj.destroy, self.context)
+
+    @mock.patch.object(objects.vnf_package.VnfPackage, 'get_by_id')
+    def test_update(self, mock_get_by_id):
+        mock_get_by_id.return_value =\
+            fakes_vnflcm.return_vnf_package_with_deployment_flavour()
+        vnf_instance_data = fakes.get_vnf_instance_data(
+            self.vnf_package.vnfd_id)
+        vnf_instance = objects.VnfInstance(context=self.context,
+                                           **vnf_instance_data)
+        vnf_instance.create()
+        id = uuidutils.generate_uuid()
+        vnf_lcm_oppccs = fakes.get_lcm_op_occs_data(
+            id,
+            vnf_instance.id)
+
+        vnf_instance.update(
+            self.context,
+            vnf_lcm_oppccs,
+            self.body_data,
+            self.vnfd_pkg_data,
+            vnf_instance_data['vnfd_id'])
