@@ -15,14 +15,15 @@
 
 import os
 import socket
-
-import mock
-from oslo_config import cfg
-import six.moves.urllib.request as urllibrequest
 import testtools
+from unittest import mock
+from urllib import request as urllibrequest
 import webob
 import webob.exc
 
+from oslo_config import cfg
+import oslo_i18n
+
 from tacker.common import exceptions as exception
 from tacker.tests import base
 from tacker import wsgi
@@ -172,7 +173,7 @@ class SerializerTest(base.BaseTestCase):
         serializer = wsgi.Serializer()
         result = serializer.deserialize(data_string, content_type)
 
-        self.assertEqual({'body': {u'servers': [u'test=pass']}}, result)
+        self.assertEqual({'body': {'servers': ['test=pass']}}, result)
 
 
 class RequestDeserializerTest(testtools.TestCase):
@@ -181,9 +182,10 @@ class RequestDeserializerTest(testtools.TestCase):
 
         class JSONDeserializer(object):
             def deserialize(self, data, action='default'):
-                return b'pew_json'
+                return 'pew_json'
 
         self.body_deserializers = {'application/json': JSONDeserializer()}
+
         self.deserializer = wsgi.RequestDeserializer(self.body_deserializers)
 
     def test_get_deserializer(self):
@@ -200,9 +202,8 @@ class RequestDeserializerTest(testtools.TestCase):
         request = wsgi.Request.blank('/')
         request.headers['Accept'] = 'application/json'
 
-        self.assertEqual(
-            'application/json',
-            self.deserializer.get_expected_content_type(request))
+        self.assertEqual('application/json',
+                         self.deserializer.get_expected_content_type(request))
 
     def test_get_action_args(self):
         """Test RequestDeserializer.get_action_args."""
@@ -214,8 +215,8 @@ class RequestDeserializerTest(testtools.TestCase):
                 'id': 12}]}
         expected = {'action': 'update', 'id': 12}
 
-        self.assertEqual(
-            expected, self.deserializer.get_action_args(env))
+        self.assertEqual(expected,
+                         self.deserializer.get_action_args(env))
 
     def test_deserialize(self):
         """Test RequestDeserializer.deserialize."""
@@ -271,9 +272,8 @@ class ResponseSerializerTest(testtools.TestCase):
     def test_get_serializer(self):
         """Test ResponseSerializer.get_body_serializer."""
         content_type = 'application/json'
-        self.assertEqual(
-            self.serializer.get_body_serializer(content_type),
-            self.body_serializers[content_type])
+        self.assertEqual(self.body_serializers[content_type],
+                         self.serializer.get_body_serializer(content_type))
 
     def test_serialize_json_response(self):
         response = self.serializer.serialize({}, 'application/json')
@@ -338,6 +338,11 @@ class RequestTest(base.BaseTestCase):
 
         self.assertEqual("application/json", result)
 
+        request = wsgi.Request.blank('/tests/123')
+        request.headers["Accept"] = ("application/json; q=0.3, ")
+        result = request.best_match_content_type()
+        self.assertEqual("application/json", result)
+
     def test_content_type_from_query_extension(self):
         request = wsgi.Request.blank('/tests/123.json')
         result = request.best_match_content_type()
@@ -368,7 +373,29 @@ class RequestTest(base.BaseTestCase):
         request.headers["Accept"] = "application/new_type"
         result = request.best_match_content_type()
 
-        self.assertEqual('application/json', result)
+        self.assertEqual("application/json", result)
+
+    def test_best_match_language(self):
+        # Test that we are actually invoking language negotiation by webop
+        request = wsgi.Request.blank('/')
+        oslo_i18n.get_available_languages = mock.MagicMock()
+        oslo_i18n.get_available_languages.return_value = [
+            'known-language', 'es', 'zh']
+        request.headers['Accept-Language'] = 'known-language'
+        language = request.best_match_language()
+        self.assertEqual('known-language', language)
+
+        # If the Accept-Leader is an unknown language, missing or empty,
+        # the best match locale should be None
+        request.headers['Accept-Language'] = 'unknown-language'
+        language = request.best_match_language()
+        self.assertIsNone(language)
+        request.headers['Accept-Language'] = ''
+        language = request.best_match_language()
+        self.assertIsNone(language)
+        request.headers.pop('Accept-Language')
+        language = request.best_match_language()
+        self.assertIsNone(language)
 
 
 class ActionDispatcherTest(base.BaseTestCase):
@@ -377,9 +404,8 @@ class ActionDispatcherTest(base.BaseTestCase):
         serializer = wsgi.ActionDispatcher()
         serializer.create = lambda x: x
 
-        self.assertEqual(
-            'pants',
-            serializer.dispatch('pants', action='create'))
+        self.assertEqual('pants',
+                         serializer.dispatch('pants', action='create'))
 
     def test_dispatch_action_None(self):
         """Test ActionDispatcher.dispatch with none action."""
@@ -387,18 +413,16 @@ class ActionDispatcherTest(base.BaseTestCase):
         serializer.create = lambda x: x + ' pants'
         serializer.default = lambda x: x + ' trousers'
 
-        self.assertEqual(
-            'Two trousers',
-            serializer.dispatch('Two', action=None))
+        self.assertEqual('Two trousers',
+                         serializer.dispatch('Two', action=None))
 
     def test_dispatch_default(self):
         serializer = wsgi.ActionDispatcher()
         serializer.create = lambda x: x + ' pants'
         serializer.default = lambda x: x + ' trousers'
 
-        self.assertEqual(
-            'Two trousers',
-            serializer.dispatch('Two', action='update'))
+        self.assertEqual('Two trousers',
+                         serializer.dispatch('Two', action='update'))
 
 
 class ResponseHeadersSerializerTest(base.BaseTestCase):
@@ -426,8 +450,8 @@ class DictSerializerTest(base.BaseTestCase):
 
     def test_dispatch_default(self):
         serializer = wsgi.DictSerializer()
-        self.assertEqual(
-            '', serializer.serialize({}, 'NonExistentAction'))
+        self.assertEqual('',
+                         serializer.serialize({}, 'NonExistentAction'))
 
 
 class JSONDictSerializerTest(base.BaseTestCase):
@@ -441,27 +465,22 @@ class JSONDictSerializerTest(base.BaseTestCase):
 
         self.assertEqual(expected_json, result)
 
-    def test_json_with_utf8(self):
-        data = b'{"a": "\xe7\xbd\x91\xe7\xbb\x9c"}'
-        as_dict = {'body': {'a': u'\u7f51\u7edc'}}
-        deserializer = wsgi.JSONDeserializer()
-        self.assertEqual(as_dict,
-                         deserializer.deserialize(data))
-
     def test_json_with_unicode(self):
-        data = b'{"a": "\u7f51\u7edc"}'
-        as_dict = {'body': {'a': u'\u7f51\u7edc'}}
-        deserializer = wsgi.JSONDeserializer()
-        self.assertEqual(as_dict,
-                         deserializer.deserialize(data))
+        input_dict = dict(servers=dict(a=(2, '\u7f51\u7edc')))
+        expected_json = b'{"servers":{"a":[2,"\\u7f51\\u7edc"]}}'
+        serializer = wsgi.JSONDictSerializer()
+        result = serializer.serialize(input_dict)
+        result = result.replace(b'\n', b'').replace(b' ', b'')
+
+        self.assertEqual(expected_json, result)
 
 
 class TextDeserializerTest(base.BaseTestCase):
 
     def test_dispatch_default(self):
         deserializer = wsgi.TextDeserializer()
-        self.assertEqual(
-            {}, deserializer.deserialize({}, 'update'))
+        self.assertEqual({},
+                         deserializer.deserialize({}, 'update'))
 
 
 class JSONDeserializerTest(base.BaseTestCase):
@@ -481,8 +500,8 @@ class JSONDeserializerTest(base.BaseTestCase):
                     'd': {'e': '1'},
                     'f': '1'}}}
         deserializer = wsgi.JSONDeserializer()
-        self.assertEqual(
-            as_dict, deserializer.deserialize(data))
+        self.assertEqual(as_dict,
+                         deserializer.deserialize(data))
 
     def test_default_raise_Malformed_Exception(self):
         """Test JsonDeserializer.default.
@@ -498,14 +517,14 @@ class JSONDeserializerTest(base.BaseTestCase):
 
     def test_json_with_utf8(self):
         data = b'{"a": "\xe7\xbd\x91\xe7\xbb\x9c"}'
-        as_dict = {'body': {'a': u'\u7f51\u7edc'}}
+        as_dict = {'body': {'a': '\u7f51\u7edc'}}
         deserializer = wsgi.JSONDeserializer()
         self.assertEqual(as_dict,
                          deserializer.deserialize(data))
 
     def test_json_with_unicode(self):
         data = b'{"a": "\u7f51\u7edc"}'
-        as_dict = {'body': {'a': u'\u7f51\u7edc'}}
+        as_dict = {'body': {'a': '\u7f51\u7edc'}}
         deserializer = wsgi.JSONDeserializer()
         self.assertEqual(as_dict,
                          deserializer.deserialize(data))
@@ -517,8 +536,8 @@ class RequestHeadersDeserializerTest(base.BaseTestCase):
         deserializer = wsgi.RequestHeadersDeserializer()
         req = wsgi.Request.blank('/')
 
-        self.assertEqual(
-            {}, deserializer.deserialize(req, 'nonExistent'))
+        self.assertEqual({},
+                         deserializer.deserialize(req, 'nonExistent'))
 
     def test_custom(self):
         class Deserializer(wsgi.RequestHeadersDeserializer):
@@ -527,44 +546,38 @@ class RequestHeadersDeserializerTest(base.BaseTestCase):
         deserializer = Deserializer()
         req = wsgi.Request.blank('/')
         req.headers['X-Custom-Header'] = 'b'
-        self.assertEqual(
-            {'a': 'b'}, deserializer.deserialize(req, 'update'))
+        self.assertEqual({'a': 'b'},
+                         deserializer.deserialize(req, 'update'))
 
 
 class ResourceTest(base.BaseTestCase):
+
+    @staticmethod
+    def my_fault_body_function():
+        return 'off'
+
+    class Controller(object):
+        def index(self, request, index=None):
+            return index
+
     def test_dispatch(self):
-        class Controller(object):
-            def index(self, request, index=None):
-                return index
-
-        def my_fault_body_function():
-            return 'off'
-
-        resource = wsgi.Resource(Controller(), my_fault_body_function)
+        resource = wsgi.Resource(self.Controller())
+        req = wsgi.Request.blank('/')
         actual = resource.dispatch(
-            resource.controller, 'index', action_args={'index': 'off'})
+            req, 'index', action_args={'index': 'off'})
         expected = 'off'
 
         self.assertEqual(expected, actual)
 
     def test_dispatch_unknown_controller_action(self):
-        class Controller(object):
-            def index(self, request, pants=None):
-                return pants
-
-        def my_fault_body_function():
-            return b'off'
-
-        resource = wsgi.Resource(Controller(), my_fault_body_function)
+        resource = wsgi.Resource(self.Controller(),
+                                 self.my_fault_body_function)
         self.assertRaises(
             AttributeError, resource.dispatch,
             resource.controller, 'create', {})
 
     def test_malformed_request_body_throws_bad_request(self):
-        def my_fault_body_function():
-            return b'off'
-
-        resource = wsgi.Resource(None, my_fault_body_function)
+        resource = wsgi.Resource(None)
         request = wsgi.Request.blank(
             "/", body=b"{mal:formed", method='POST',
             headers={'Content-Type': "application/json"})
@@ -573,9 +586,7 @@ class ResourceTest(base.BaseTestCase):
         self.assertEqual(400, response.status_int)
 
     def test_wrong_content_type_throws_unsupported_media_type_error(self):
-        def my_fault_body_function():
-            return b'off'
-        resource = wsgi.Resource(None, my_fault_body_function)
+        resource = wsgi.Resource(None)
         request = wsgi.Request.blank(
             "/", body=b"{some:json}", method='POST',
             headers={'Content-Type': "xxx"})
@@ -583,29 +594,20 @@ class ResourceTest(base.BaseTestCase):
         response = resource(request)
         self.assertEqual(400, response.status_int)
 
-    def test_wrong_content_type_server_error(self):
-        def my_fault_body_function():
-            return b'off'
-        resource = wsgi.Resource(None, my_fault_body_function)
+    def test_wrong_content_type_bad_request_error(self):
+        resource = wsgi.Resource(self.Controller())
         request = wsgi.Request.blank(
             "/", method='POST', headers={'Content-Type': "unknow"})
 
         response = resource(request)
-        self.assertEqual(500, response.status_int)
+        self.assertEqual(400, response.status_int)
 
     def test_call_resource_class_bad_request(self):
-        class Controller(object):
-            def index(self, request, index=None):
-                return index
-
-        def my_fault_body_function():
-            return b'off'
-
         class FakeRequest(object):
             def __init__(self):
                 self.url = 'http://where.no'
                 self.environ = 'environ'
-                self.body = b'body'
+                self.body = 'body'
 
             def method(self):
                 pass
@@ -613,39 +615,25 @@ class ResourceTest(base.BaseTestCase):
             def best_match_content_type(self):
                 return 'best_match_content_type'
 
-        resource = wsgi.Resource(Controller(), my_fault_body_function)
+        resource = wsgi.Resource(self.Controller())
         request = FakeRequest()
         result = resource(request)
-        self.assertEqual(400, result.status_int)
+        self.assertEqual(415, result.status_int)
 
     def test_type_error(self):
-        class Controller(object):
-            def index(self, request, index=None):
-                return index
-
-        def my_fault_body_function():
-            return b'off'
-        resource = wsgi.Resource(Controller(), my_fault_body_function)
+        resource = wsgi.Resource(self.Controller())
         request = wsgi.Request.blank(
-            "/", method='POST', headers={'Content-Type': "json"})
+            "/", method='GET', headers={'Content-Type': "json"})
 
-        response = resource.dispatch(
-            request, action='index', action_args='test')
+        response = resource(request)
         self.assertEqual(400, response.status_int)
 
-    def test_call_resource_class_internal_error(self):
-        class Controller(object):
-            def index(self, request, index=None):
-                return index
-
-        def my_fault_body_function():
-            return b'off'
-
+    def test_call_resource_class_bad_request_error(self):
         class FakeRequest(object):
             def __init__(self):
                 self.url = 'http://where.no'
                 self.environ = 'environ'
-                self.body = b'{"Content-Type": "json"}'
+                self.body = '{"Content-Type": "json"}'
 
             def method(self):
                 pass
@@ -653,10 +641,10 @@ class ResourceTest(base.BaseTestCase):
             def best_match_content_type(self):
                 return 'application/json'
 
-        resource = wsgi.Resource(Controller(), my_fault_body_function)
+        resource = wsgi.Resource(self.Controller())
         request = FakeRequest()
         result = resource(request)
-        self.assertEqual(500, result.status_int)
+        self.assertEqual(400, result.status_int)
 
 
 class MiddlewareTest(base.BaseTestCase):
@@ -672,14 +660,16 @@ class MiddlewareTest(base.BaseTestCase):
 class FaultTest(base.BaseTestCase):
     def test_call_fault(self):
         class MyException(object):
-            status_int = 415
+            code = 415
             explanation = 'test'
 
-        my_exceptions = MyException()
-        my_fault = wsgi.Fault(exception=my_exceptions)
-        request = wsgi.Request.blank(
-            "/", method='POST', headers={'Content-Type': "unknow"})
-        response = my_fault(request)
+        my_exception = MyException()
+        converted_exp = exception.ConvertedException(code=my_exception.code,
+                                    explanation=my_exception.explanation)
+        my_fault = wsgi.Fault(converted_exp)
+        req = wsgi.Request.blank("/", method='POST',
+                                 headers={'Content-Type': "unknow"})
+        response = my_fault(req)
         self.assertEqual(415, response.status_int)
 
 
